#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "shd:i:"

/*
 * if not supplied, how many seconds to wait for event
 * and the minimum time we will allow to request
 * (since we use sleep() with a second granularity to
 * accuate the event)
 */
#define DEFAULT_WAIT_SEC    10
#define MIN_WAIT_SEC        2

/* 
 * for ultra sparc .. appears that it's
 * 100 ticks per second in default configuration
 */
#define TICKS_PER_SEC       1000                 


/********************************************************
**
** Test Module: DoAll.c
**
** Purpose:     Issue multiple tests to the TPRO/TSAT
**              PCI board 
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_AltObj     altitude;
    TPRO_DateObj    dateInfo;
    TPRO_WaitObj    WaitObj;
    TPRO_LongObj    longObj;
    TPRO_LatObj     latObj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             secToWait = DEFAULT_WAIT_SEC;
    int             secToEvent = 0;
    char            firmwareVer [TPRO_FIRMWARE_LENGTH + 1];
    char            drvVersion [TPRO_DRV_VERSION_LENGTH + 1];
    int             isSat = 0;
    int             tidx;
    char            *tstName;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 's' :
                isSat = 1;
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

while ( 1 )
{
    if ( isSat )
    {
        /*
         * get the altitude the device
         */
        pgmname = "GetAltitude";

        if ( (rc = TPRO_getAltitude(hnd, &altitude)) == TPRO_SUCCESS )
        {
            (void) printf("<%s> %s  <%f> meters\n", 
                fullDevName, pgmname,  altitude.meters );
            Sleep( 1000 );  // sleep for millisecond  (.001 sec)
        }
        else
        {
            goto ERR_DONE; 
        }
    }

    /* get the date of the device
     */
    pgmname = "GetDate";

    if ( (rc = TPRO_getDate(hnd, &dateInfo)) == TPRO_SUCCESS )
    {
        /* success:  print out the date
         */
        (void)printf("<%s> %s:  Year <%d> Month <%d> Day <%d>\n", 
                    fullDevName, pgmname,
                    dateInfo.year, dateInfo.month, dateInfo.day);
        Sleep( 1 );  // sleep for millisecond  (.001 sec)
    }
    else
    {
        goto ERR_DONE; 
    }

    pgmname = "WaitEvent";

    /* calc the time use to sleep before issuing the
     * simulated event.
     */
    secToEvent = (secToWait / 2) ;



            (void) printf("Waiting for time event ...\n");

            WaitObj.ticks = (secToWait * TICKS_PER_SEC) ;

            if ( (rc = TPRO_waitEvent(hnd, &WaitObj)) != TPRO_SUCCESS )
            {
                char emsg[MAX_ERR_MESG_LENGTH];

                switch (rc)
                {
                    case TPRO_TIMEOUT_ERR:
                        (void)sprintf_s(emsg, MAX_ERR_MESG_LENGTH, " TIMEOUT_ERROR <%d> sec ", secToWait );
                        errMsg = emsg;
                        break;
                    case TPRO_HANDLE_ERR:
                        errMsg = "TPRO_HANDLE_ERR";
                        break;
                    case TPRO_COMM_ERR:
                        errMsg = "TPRO_COMM_ERR";
                        break;
                    default:
                        errMsg = "Unknown error";
                        break;
                }

                (void) printf("\n%s: <%s> %s rc: <%d>.\n", 
                            pgmname, fullDevName, errMsg, rc);

                if ( rc != TPRO_TIMEOUT_ERR )
                {
                    ret = 1;
                    rc = TPRO_SUCCESS;
                    goto ERR_DONE;
                }
            }
            else
            {
                /*
                 * success notification
                 */
                (void)printf(" received Days <%d> Hours <%d> Min <%d> Sec <%f>\n",
                                WaitObj.days, WaitObj.hours, 
                                WaitObj.minutes, WaitObj.seconds);
                Sleep( 1000 );  // sleep for millisecond  (.001 sec)

            }


    /* 
     * Display the TPRO/TSAT board options 
     */
    (void)printf("<%s> PCI SubsysID: 0x<%04x> PCI devID 0x<%4x>\n",
                    fullDevName, hnd->options, hnd->devid );

    /*
     * perform two tests to get both driver and firmware version
     */
    for ( tidx = 0 ; tidx < 1 ; tidx++ )
    {
        /*
         * perform the appropriate get operation
         */
        if (tidx == 0)
        {
            tstName = "DRV Version";
            pgmname = "DRV Version";
            rc = TPRO_getDriver(hnd, drvVersion);
        }
        else
        {
            // SKIP THIS... AFFECTS BOARD PERFORMANCE / SYNC
            //
            tstName = "FRMWARE Version";
            pgmname = "FRMWARE Version";
            rc = TPRO_getFirmware(hnd, firmwareVer);
        }

        if ( rc != TPRO_SUCCESS )
        {
            ret = 1;
            goto ERR_DONE; 
        }
        else
        {
            /*
             * success:  output driver or firmware version info
             */
            if (tidx == 0 )
            {
                (void)printf("<%s> %s: <%s>\n", fullDevName, tstName,  drvVersion);
            }
            else
            {
                (void)printf("<%s> %s: < %02X-%02X-%02X-%02X >\n", 
                        fullDevName, tstName, 
                        (uint8_t)firmwareVer[0], (uint8_t)firmwareVer[1], 
                        (uint8_t)firmwareVer[2], (uint8_t)firmwareVer[3] );
            }
            Sleep( 1000 );  // sleep for millisecond  (.001 sec)
        }
    }

    if ( isSat )
    {
        /*
         * loop through and get the latitude and longitude
         */
        unsigned short  tstDegrees;
        float           tstMinutes;
        for ( tidx = 0 ; tidx < 2 ; tidx++ )
        {
            /*
             * perform the appropriate get operation
             */
            if (tidx == 0)
            {
                tstName = "Latitude";
                pgmname = "Latitude";
                rc = TPRO_getLatitude(hnd, &latObj);
                tstDegrees = latObj.degrees;
                tstMinutes = latObj.minutes;
            }
            else
            {
                tstName = "Longitude";
                pgmname = "Longitude";
                rc = TPRO_getLongitude(hnd, &longObj);
                tstDegrees = longObj.degrees;
                tstMinutes = longObj.minutes;
            }

            if ( rc != TPRO_SUCCESS )
            {
                ret = 1;
                goto ERR_DONE; 
            }
            else
            {
                /*
                 * success:  print out latitude or longitude
                 */
                (void)printf("<%s> %s: <%d> degrees <%f> minutes\n", 
                            fullDevName, tstName, tstDegrees, tstMinutes);
                Sleep( 1000 );  // sleep for millisecond  (.001 sec)
            }
        }
    }
    else
    {
        // Non sat board... more  date getting
        int cnt;
        pgmname = "GetDate";
        for ( cnt = 0 ; cnt < 2 ; cnt++ )
        {

            if ( (rc = TPRO_getDate(hnd, &dateInfo)) == TPRO_SUCCESS )
            {
                /* success:  print out the date
                 */
                (void)printf("<%s> %s:  Year <%d> Month <%d> Day <%d>\n",
                            fullDevName, pgmname,
                            dateInfo.year, dateInfo.month, dateInfo.day);
                Sleep( 1000 );  // sleep for millisecond  (.001 sec)
            }
            else
            {
                goto ERR_DONE;
            }
        }
    }
 }

ERR_DONE:

    if ( rc != TPRO_SUCCESS )
    {
        (void) printf("%s: <%s>  failure - rc: <%d>.\n",
                    pgmname, fullDevName, rc);
        ret = 1;
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-s] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-s   specify a TSAT board\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}

